# Copyright 2020 The SODA Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from delfin import context
from delfin import test
from delfin.api import middlewares
from delfin.wsgi import common


class TestContextWrapper(test.TestCase):

    def _get_fake_req(self):
        env = {}
        req = common.Request(env)
        return req

    def test_context_wrapper(self):
        context_wrapper = middlewares.ContextWrapper(common.Application())
        req = self._get_fake_req()
        context_wrapper(req)

        self.assertIsInstance(req.environ['delfin.context'],
                              context.RequestContext)
